@@ -1,8 +1,8 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from trytond.model import (ModelStorage, ModelView, DeactivableMixin, fields,
-    tree)
-from trytond.pool import PoolMeta, Pool
+from trytond.model import (
+    DeactivableMixin, ModelStorage, ModelView, fields, tree)
+from trytond.pool import Pool, PoolMeta
 
 
 class ClassificationMixin(DeactivableMixin):
@@ -29,11 +29,22 @@ def classification_tree(name):
 class Template(metaclass=PoolMeta):
     __name__ = 'product.template'
 
-    classification = fields.Reference('Classification',
-        selection='get_classification',
-        domain=[
-            ('selectable', '=', True),
-            ])
+    classification = fields.Reference(
+        "Classification", selection='get_classification')
+
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        for model in cls._get_classification():
+            if model in cls.classification.domain:
+                cls.classification.domain[model] = [
+                    cls.classification.domain[model],
+                    ('selectable', '=', True),
+                    ]
+            else:
+                cls.classification.domain[model] = [
+                    ('selectable', '=', True),
+                    ]
 
     @classmethod
     def _get_classification(cls):
@@ -44,11 +55,9 @@ class Template(metaclass=PoolMeta):
     def get_classification(cls):
         pool = Pool()
         Model = pool.get('ir.model')
+        get_name = Model.get_name
         models = cls._get_classification()
-        models = Model.search([
-                ('model', 'in', models),
-                ])
-        return [(None, '')] + [(m.model, m.name) for m in models]
+        return [(None, '')] + [(m, get_name(m)) for m in models]
 
 
 class Product(metaclass=PoolMeta):
